import QtQuick 1.1
import Process 0.1
import FileIO 1.0

import ".."
import "../Keypad"

Item{
    id: root
    anchors.fill: parent

    property int fontSize: 24
    property int topOffset: -150
    property int itemSpacing: 12
    property int itemColumnSpacing: 50
    property bool debug: false

    property string ip
    property string broadcast
    property string mask
    property bool isWireless: false
    property string currentESSID: qsTr("not connected","wireless network")
    property bool success: false
    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded

    onIsUsbDeviceLoadedChanged: print("USB DEVICE: " + isUsbDeviceLoaded)

    function debugLog(string) {
        if(root.debug)
            print("network: " + string)
    }

    function parseString(str, name)
    {
        // ifconfig still returns output for interfaces that are down, so filter here to
        // make sure it is up before continuing.
        if (str.indexOf("UP") === -1) {
            return
        }

        var ipSearch = str.match(/inet addr:(.+?) /)
        if (ipSearch === null) {
            return // no IP found. Abort and do not update values
        }

        root.ip = ipSearch[1]
        root.broadcast = str.match(/Bcast:(.+?) /)[1]
        root.mask = str.match(/Mask:(.+?)\s/)[1]
        if (name === "wlan0") {
            root.isWireless = true
        }
        root.success = true
    }

    function resetFields() {
        root.isWireless = false
        root.success = false
        root.ip = ""
        root.broadcast = ""
        root.mask = ""
    }

    function updateFields() {
        resetFields()
        debugLog("sending command... "+ifconfigProcess.program)
        ifconfigProcess.start()
        iwconfigProcess.start()
        netpulseEnabledMode.value = options.netpulseEnabled ? true : false
        browserEnabledMode.value = options.browserEnabled ? true : false
    }

    Component.onCompleted: {
        updateFields()
        homeUrlEdit.text = options.homeURL
        linksUrlEdit.text = options.linksURL
    }

    Process{
        id: ifconfigProcess
        program: "ifconfig"
        arguments: ["eth0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "eth0")
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }

        onFinished: {
            if (!root.success) {
                ifconfigProcessWireless.start()
            }
        }
    }

    Process {
        id: iwconfigProcess
        program: "iwconfig"
        arguments: ["wlan0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "iwconfig output: " + output)
            if(output.match(/wlan0/)) {
                var essidSearch = output.match(/ESSID:"(.+?)"\s/)
                root.currentESSID = essidSearch ? essidSearch[1] : qsTr("not connected","wireless network")
            }
        }
    }

    Process {
        id: configureNetworkProcess

        onStarted: resetFields()
        onFinished: { updateFields(); dataExport.getNetworkAddresses() }

        program: "scripts/configure_network.sh"
        arguments: ["eth0"]
    }

    Process{
        id: ifconfigProcessWireless
        program: "ifconfig"
        arguments: ["wlan0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "wlan0")
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }
    }

    Grid {
        id: labels

        columns: 2
        spacing: itemSpacing
        anchors { left: parent.left ; leftMargin: 200 ; right: parent.right ; rightMargin: 200 }
        anchors { verticalCenter: parent.verticalCenter ;  verticalCenterOffset: topOffset }

        // Interface: xxx   [configure]
        Text {
            text: qsTr("Interface:") + application.emptyString
            font.pointSize: fontSize
            width: paintedWidth
        }
        Row {
            spacing: 170

            Text {
                id: accessPoint
                text: root.isWireless ? root.currentESSID : qsTr("Ethernet")
                font.pointSize: fontSize
            }

            ImageButton {
                id: interfaceButton
                width: 70
                height: 50

                label_under: true
                label_color: "black"
                upimage: "graphics/205/SM_Button_Up.png"
                downimage: "graphics/205/SM_Button_Down.png"
                label: qsTr("setup connection")
                onClicked: {
                    wifiBrowser.visible = true
                }
            }
        }

        // IP Address: xxx
        Text{
            text: qsTr("IP Address") + application.emptyString
            horizontalAlignment: Text.AlignLeft
            width: paintedWidth
            font.pointSize: fontSize
        }
        Text{
            text: root.ip ? root.ip : qsTr("No Network Detected") + application.emptyString
            horizontalAlignment: Text.AlignRight
            width: paintedWidth
            font.pointSize: fontSize
        }

        // Subnet Mask: xxx
        Text{
            text: qsTr("Subnet Mask") + application.emptyString
            font.pointSize: fontSize
        }
        Text{
            text: root.mask ? root.mask : qsTr("No Network Detected") + application.emptyString
            font.pointSize: fontSize
        }

        // MAC Address: xxx
        Text{
            text: qsTr("MAC Address") + application.emptyString
            font.pointSize: fontSize
        }
        Text{
            text: root.mask ? dataExport.macAddress : qsTr("No Network Detected") + application.emptyString
            font.pointSize: fontSize
        }

        // Bluetooth MAC
        Text{
            text: qsTr("Bluetooth Address") + application.emptyString
            font.pointSize: fontSize
        }
        Text{
            text: bluetoothAdapter.address ? bluetoothAdapter.address : qsTr("Not Installed") + application.emptyString
            font.pointSize: fontSize
        }
    }

    Row {
        visible: main.isTouch
        id: netpulseSetupControls

        anchors { top: labels.bottom ; topMargin: 30 ; left: labels.left }

        spacing: 150

        ToggleButtonLabeled{
            id: netpulseEnabledMode
            labelString: qsTr("Netpulse Enabled") + application.emptyString
            onValueChanged:
            {
                options.netpulseEnabled = value ? true : false
            }
        }

        ImageButton {
            id: netpulseInstallButton

            label: qsTr("Configure Netpulse") + application.emptyString
            label_under: true
            label_color: "black"
            upimage: "graphics/205/SM_Button_Up.png"
            downimage: "graphics/205/SM_Button_Down.png"

            onClicked: {
                netpulseInstall.show()
                debugLog("showing netpulse install page")
            }
        }
    }

    ToggleButtonLabeled{
        id: browserEnabledMode

        anchors { top: netpulseSetupControls.bottom ; topMargin: 30 ; left: labels.left }
        labelString: qsTr("Web Browser Enabled") + application.emptyString
        onValueChanged:
        {
            options.browserEnabled = value ? true : false
        }
    }

    AlphaKeypad {
        id: networkKeypad
        visible: false
        anchors { horizontalCenter: parent.horizontalCenter ; bottom: parent.bottom }
        z: 9999
        onClose: hide()
    }

    Row {
        id: homeUrlRow

        anchors { top: browserEnabledMode.bottom ; topMargin: 30 ; left: labels.left ; right: parent.right ; rightMargin: 20 }

        spacing: 5
        visible: options.browserEnabled

        Text {
            id: urlLabel

            text: qsTr("Homepage:")
            width: paintedWidth
            height: paintedHeight
        }

        TextInput {
            id: homeUrlEdit

            width: parent.width - urlLabel.width - 10

            Rectangle { color: "white" ; border.color: "black" ; anchors.fill: parent ; z: parent.z-1 }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    networkKeypad.setValue(homeUrlEdit.text)
                    networkKeypad.targetItem = homeUrlEdit
                    networkKeypad.show()
                }
            }
        }

        // get url result typed in alpha keypad
        Connections {
            target: networkKeypad.targetItem === homeUrlEdit ? networkKeypad : null
            ignoreUnknownSignals: true

            onValueEntered: {
                networkKeypad.hide()
                homeUrlEdit.text = val
                networkKeypad.clear()
                networkKeypad.targetItem = null
                options.homeURL = homeUrlEdit.text
            }
        }
    }

    Row {
        id: linksUrlRow

        anchors { top: homeUrlRow.bottom ; topMargin: 20 ; left: labels.left ; right: parent.right ; rightMargin: 20 }

        spacing: 5
        visible: options.browserEnabled

        Text {
            id: linksUrlLabel

            text: qsTr("Links page:")
            width: paintedWidth
            height: paintedHeight
        }

        TextInput {
            id: linksUrlEdit

            width: parent.width - linksUrlLabel.width - 10

            Rectangle { color: "white" ; border.color: "black" ; anchors.fill: parent ; z: parent.z-1 }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    networkKeypad.setValue(linksUrlEdit.text)
                    networkKeypad.targetItem = linksUrlEdit
                    networkKeypad.show()
                }
            }
        }

        // get url result typed in alpha keypad
        Connections {
            target: networkKeypad.targetItem === linksUrlEdit ? networkKeypad : null
            ignoreUnknownSignals: true

            onValueEntered: {
                networkKeypad.hide()
                linksUrlEdit.text = val
                networkKeypad.clear()
                networkKeypad.targetItem = null
                options.linksURL = linksUrlEdit.text
            }
        }
    }

    Text {
        id: reconfiguringNotice

        text: qsTr("Reconfiguring Network Settings...") + application.emptyString
        visible: configureNetworkProcess.processState === Process.Running
        font.pointSize: fontSize
        width: paintedWidth
        anchors.horizontalCenter: parent.horizontalCenter
    }

    WifiBrowser {
        id: wifiBrowser

        visible: false
        onSelected: {
            visible = false

            var auth
            if(accessPoint.encrypted && accessPoint.auth) {
                auth = (accessPoint.auth === "PSK") ? "WPA-PSK" : "NONE"
            }
            else
                auth = "NONE"

            var args = [ accessPoint.interface, accessPoint.essid, accessPoint.password, auth]
            configureNetworkProcess.arguments = args
            if(configureNetworkProcess.processState != Process.NotRunning)
                configureNetworkProcess.terminate()
            configureNetworkProcess.start()
        }
        onCanceled: visible = false
        anchors { horizontalCenter: parent.horizontalCenter ; verticalCenter: parent.verticalCenter }
    }
}
