import QtQuick 1.1
import Process 0.1
import VideoOverlay 1.0
import VLCVideo 1.0
import "Television/"

// This element represents the television tuner and its channel scan results.

Item {
    id: root

    property bool on: false
    property string mrl
    property real aspect: videoPlayer.aspect > 0 ? videoPlayer.aspect : 1.333
    property bool playing: videoPlayer.playing

    property alias player: videoPlayer
    property alias fullscreen: videoPlayer.fullscreen
    property bool tuning: videoPlayer.tuning
    property bool muted: false
    property alias channelList: chanlist
    property int currentChannel: -1
    property string tunerType
    property string scriptsdir: "/usr/local/bin/truetouch/scripts"
    property string channelFile: "/data/tv_conf.xml"
    property int numCaptionTracks: videoPlayer.ccTrackCount
    property alias captionTrack: videoPlayer.captionTrack
    property bool tuningTimedOut: false
    property bool isAuxInput: false
    property bool isIpodInput: false

    signal clicked()
    signal tuningTimeout()

    visible: false

    function tuneChannelID(id) {
        if(id == "AUXINPUT") {
            root.isAuxInput = true
        }
        else if(id == "IPODINPUT") {
            //special case for IPODINPUT - an AUX channel not in the channel list
            root.isIpodInput = true
            root.currentChannel = -1
            root.mrl = chanlist.getAuxMrl("NTSC_M")
            return
        }
        else {
            root.isAuxInput = false
            root.isIpodInput = false
        }
        var idx = chanlist.findChannel(id)
        if(idx >= 0) {
            root.currentChannel = idx
            root.on = true
        }
    }

    function channelUp() {
        var idx = root.currentChannel

        if(idx < chanlist.channels.count-1) {
            idx++
        }
        else
            idx = 0

        var chanid = chanlist.channels.get(idx).id

        if(chanid) {
            tuneChannelID(chanid)
        }
    }

    function channelDown() {
        var idx = root.currentChannel
        if(idx > 0) {
            idx--
        }
        else
            idx = channelList.channels.count-1

        var chanid = chanlist.channels.get(idx).id
        if(chanid) {
            root.on = true
            tuneChannelID(chanid)
        }
    }

    // switch off the TV
    function stop() {
        root.on = false
    }

    Component.onCompleted: {
        tunerDetect.start()
        chanlist.readChannels(root.channelFile)
    }

    onOnChanged: {
        print("TV " + (on ? "on" : "off"))
        if (on) {
            root.mrl = getTVMrl()
            videoPlayer.play()
        }
        else {
            videoPlayer.stop()
        }
    }

    // get the current channel mrl, given the selected channel number
    function getTVMrl() {
        if( root.channelList.channels.get(currentChannel) )
            return chanlist.mrl(currentChannel)
        else
            return ""
    }

    function getTVChannelName() {
        if( root.channelList.channels.get(currentChannel) )
            return root.channelList.channels.get(currentChannel).name
        else
            return ""
    }

    function getTVChannelNumber() {
        var item = root.channelList.channels.get(currentChannel)
        if( item )
            return item.number
        else
            return ""
    }

    onCurrentChannelChanged: {
        if(!on)
            return
        root.mrl = getTVMrl()
    }

    onMrlChanged: {
        print("TV MRL:" + mrl)
        videoPlayer.mrl = mrl
    }

    ChannelList {
        id: chanlist

        channelFile: root.channelFile
    }

    onTuningChanged: {
        if(root.tuning) {
            tuningTimer.restart()
            root.tuningTimedOut = false
        }
        else {
            tuningTimer.stop()
        }
    }

    Timer {
        id: tuningTimer
        interval: 7000

        onTriggered: {
            root.tuningTimedOut = true
            root.tuningTimeout()
            videoPlayer.stop()
            videoPlayer.play()
        }
    }

    VLCVideo {
        id: videoPlayer
        objectName: "television"

        property bool fullscreen  // set to true to expand into fullscreenFrame

        commandLineArgs: {
            var args = ["--video-filter=deinterlace", "--deinterlace=-1", "--v4l2-use-libv4l2" ]
            if(options.tvHwAccel)
                args = args.concat(["--ffmpeg-hw"])
            if(options.tvNoFrameDrop)
                args = args.concat(["--no-drop-late-frames", "--ffmpeg-skip-frame=1"])
            return args
        }

        onMediaError: root.tuningTimedOut = true

        debug: false
        // don't stream audio from tuner if it is already coming from RCA jacks or iPod
        audioDevice: (root.isAuxInput || root.isIpodInput) ? "" : "hw:1,0"
        deinterlace: false
        muted: root.muted
    }

    Process { id: tunerDetect
        program: root.scriptsdir + "/tunertype.sh"

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
			root.tunerType = output.trim()
            print( "tuner type: " + output)
        }
    }
}
